// Copyright 2012, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Hannah Bast <bast>.

import java.text.DecimalFormat;

/**
 * An item from a linked list. Values can be arbitrarty objects.  For
 * simplicity, make all member variables public, so that we can access them
 * directly from everywhere and need not write explicit getter and setter
 * functions for each.
 * 
 * @param <T> The type of the elements in the list.
 */
public class LinkedListItem<T> {

  /**
   * Create list item with given value.
   */
  public LinkedListItem(T v) {
    this.value = v;
  }

  /**
   * The value of this element.
   */
  public T value;

  /**
   * Pointer to the next item in the list. Is null, if this is the last item.
   */
  public LinkedListItem<T> next;

  /**
   * Pointer to the previous item in the list. Is null, if this is the first
   * item.
   */
  public LinkedListItem<T> previous;

  /**
   * Hash code of a LinkedList item as fixed-size string with leading zeroes. If
   * item is null, return all zeroes.
   */
  public static String id(LinkedListItem<?> item) {
    DecimalFormat formatter = new DecimalFormat("0000000000");
    return formatter.format(item == null ? 0 : item.hashCode());
  }

  /**
   * List item in human-readable form (for debugging). Shows the pointer as the
   * hash code of the respective LinkedListItem objects.
   */
  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append("[this=" + LinkedListItem.id(this)
              + ",prev=" + LinkedListItem.id(this.previous)
              + ",next=" + LinkedListItem.id(this.next) + "]");
    return sb.toString();
  }
}
